VERSION 5.00
Begin VB.Form Form1 
   Caption         =   "ListBox IntegralHeight demo"
   ClientHeight    =   4230
   ClientLeft      =   3435
   ClientTop       =   4290
   ClientWidth     =   4845
   ClipControls    =   0   'False
   LinkTopic       =   "Form1"
   ScaleHeight     =   4230
   ScaleWidth      =   4845
   Begin VB.CheckBox Check1 
      Caption         =   "Override the listbox's IntegralHeight property"
      Height          =   405
      Left            =   240
      TabIndex        =   1
      Top             =   540
      Width           =   3975
   End
   Begin VB.ListBox List1 
      Height          =   2205
      Left            =   360
      TabIndex        =   0
      Top             =   1320
      Width           =   3435
   End
   Begin VB.Label Label1 
      Caption         =   "List1.IntegralHeight = "
      Height          =   405
      Left            =   240
      TabIndex        =   2
      Top             =   60
      Width           =   3615
   End
End
Attribute VB_Name = "Form1"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
Option Explicit
'
' Brad Martinez http://www.mvps.org/ccrp
'
' Demonstrates how to both override the ListBox's IntegralHeight property
' behavior, and how to implement your own integral height-like algorithm.

Private m_hwndLB As Long   ' the ListBox's window handle
Private m_yLB As Long          ' the ListBox's static top coordinate
'

Private Sub Form_Load()
  Dim i As Integer
  
  ScaleMode = vbPixels
  Move (Screen.Width - Width) * 0.5, (Screen.Height - Height) * 0.5
  
  m_hwndLB = List1.hWnd
  m_yLB = Check1.Top + Check1.Height + 10
  
  Label1 = "List1.IntegralHeight = " & List1.IntegralHeight & _
                 " (change in designtime to observe alternate behavior)"
  
  For i = 1 To 25: List1.AddItem i: Next
  
End Sub

Private Sub Check1_Click()
  ' Re-adjust the ListBox's height accordingly.
  Call Form_Resize
End Sub

Private Sub Form_Resize()
  
  If (WindowState <> vbMinimized) And (ScaleHeight > m_yLB) Then
    
    ' If not overriding the ListBox's IntegralHeight prop setting...
    If (Check1 = vbUnchecked) Then
      
      ' Let the ListBox do its normal thing...
      List1.Move 0, m_yLB, ScaleWidth, ScaleHeight - m_yLB
    
    Else
      
      ' We are overriding the ListBox's IntegralHeight prop setting.
      
      ' If the the ListBox's IntegralHeight prop is set...
      If List1.IntegralHeight Then
        ' Subclass the ListBox so we can override the its IntegralHeight
        ' behavior, move it accordingly, and unsubclass it.
        Call SubClass(m_hwndLB, AddressOf WndProc)
        List1.Move 0, m_yLB, ScaleWidth, ScaleHeight - m_yLB
        Call UnSubClass(m_hwndLB)
        
      Else
        ' The the ListBox's IntegralHeight prop is not set, implement
        ' our own IntegralHeight behavior.
        List1.Move 0, m_yLB, ScaleWidth, GetLBItegralHeight(ScaleHeight - m_yLB)
        
      End If   ' List1.IntegralHeight
    End If   ' (Check1 = vbUnchecked)
  End If   ' (WindowState <> vbMinimized) And (ScaleHeight > m_yLB)

End Sub

Private Function GetLBItegralHeight(cyLBOld As Long) As Long
  Dim cyLBItem As Long      ' ListBox item height
  Dim rcLB As RECT            ' ListBox's window rect
  Dim rcLBClient As RECT   ' ListBox's client rect
  Dim cyLBNC As Long        ' combined height of the ListBox's non-client window borders
    
  ' Get the height of the ListBox items (assumes that all of the ListBox items
  ' are the same height [that LBS_OWNERDRAWVARIABLE is not set]).
  cyLBItem = SendMessage(m_hwndLB, LB_GETITEMHEIGHT, 0, 0)
  
  ' Go with a common defualt height if something went wrong...
  If cyLBItem = 0 Then cyLBItem = 13
  
  ' Get the ListBox's current window and client rects.
  Call GetWindowRect(m_hwndLB, rcLB)
  Call GetClientRect(m_hwndLB, rcLBClient)
  
  ' Calculate the combined height of the ListBox's non-client window borders
  ' (the difference between the ListBox's client and window heights)
  cyLBNC = (rcLB.Bottom - rcLB.Top) - (rcLBClient.Bottom - rcLBClient.Top)
  
  ' Reurn the ListBox's new window height as a multiple of its
  ' items' height (i.e. integral height).
  GetLBItegralHeight = (cyLBItem * ((cyLBOld - cyLBNC) \ cyLBItem)) + cyLBNC

End Function
