Attribute VB_Name = "mWndProc"
Option Explicit

' Brad Martinez http://www.mvps.org/ccrp

' Code was written in and formatted for 8pt MS San Serif

' The NMHDR structure contains information about a notification message. The pointer
' to this structure is specified as the lParam member of the WM_NOTIFY message.
Public Type NMHDR
  hwndFrom As Long   ' Window handle of control sending message
  idFrom As Long        ' Identifier of control sending message
  code  As Long          ' Specifies the notification code
End Type

Private Const WM_NOTIFY = &H4E
Private Const WM_DESTROY = &H2

Private Declare Function GetProp Lib "user32" Alias "GetPropA" (ByVal hWnd As Long, ByVal lpString As String) As Long
Private Declare Function SetProp Lib "user32" Alias "SetPropA" (ByVal hWnd As Long, ByVal lpString As String, ByVal hData As Long) As Long
Private Declare Function RemoveProp Lib "user32" Alias "RemovePropA" (ByVal hWnd As Long, ByVal lpString As String) As Long

Declare Sub MoveMemory Lib "kernel32" Alias "RtlMoveMemory" (pDest As Any, pSource As Any, ByVal dwLength As Long)

Public Enum GWL_nIndex
  GWL_WNDPROC = (-4)
'  GWL_HWNDPARENT = (-8)
  GWL_ID = (-12)
  GWL_STYLE = (-16)
  GWL_EXSTYLE = (-20)
'  GWL_USERDATA = (-21)
End Enum

Declare Function GetWindowLong Lib "user32" Alias "GetWindowLongA" (ByVal hWnd As Long, ByVal nIndex As GWL_nIndex) As Long
Declare Function SetWindowLong Lib "user32" Alias "SetWindowLongA" (ByVal hWnd As Long, ByVal nIndex As GWL_nIndex, ByVal dwNewLong As Long) As Long

Private Declare Function CallWindowProc Lib "user32" Alias "CallWindowProcA" (ByVal lpPrevWndFunc As Long, ByVal hWnd As Long, ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
Private Declare Function DefWindowProc Lib "user32" Alias "DefWindowProcA" (ByVal hWnd As Long, ByVal wMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long

Private Const OLDWNDPROC = "OldWndProc"
Private Const OBJECTPTR = "ObjectPtr"

#If DEBUGWINDOWPROC Then
  ' maintains a WindowProcHook reference for each subclassed window.
  ' the window's handle is the collection item's key string.
  Private m_colWPHooks As New Collection
#End If
'

Public Function SubClass(hWnd As Long, _
                                         lpfnNew As Long, _
                                         Optional objNotify As Object = Nothing) As Boolean
  Dim lpfnOld As Long
  Dim fSuccess As Boolean
  On Error GoTo Out
  
  If GetProp(hWnd, OLDWNDPROC) Then
    SubClass = True
    Exit Function
  End If
  
#If (DEBUGWINDOWPROC = 0) Then
    lpfnOld = SetWindowLong(hWnd, GWL_WNDPROC, lpfnNew)

#Else
    Dim objWPHook As WindowProcHook
    
    Set objWPHook = CreateWindowProcHook
    m_colWPHooks.Add objWPHook, CStr(hWnd)
    
    With objWPHook
      Call .SetMainProc(lpfnNew)
      lpfnOld = SetWindowLong(hWnd, GWL_WNDPROC, .ProcAddress)
      Call .SetDebugProc(lpfnOld)
    End With

#End If
  
  If lpfnOld Then
    fSuccess = SetProp(hWnd, OLDWNDPROC, lpfnOld)
    If (objNotify Is Nothing) = False Then
      fSuccess = fSuccess And SetProp(hWnd, OBJECTPTR, ObjPtr(objNotify))
    End If
  End If
  
Out:
  If fSuccess Then
    SubClass = True
  
  Else
    If lpfnOld Then Call SetWindowLong(hWnd, GWL_WNDPROC, lpfnOld)
    MsgBox "Error subclassing window &H" & Hex(hWnd) & vbCrLf & vbCrLf & _
                  "Err# " & Err.Number & ": " & Err.Description, vbExclamation
  End If
  
End Function

Public Function UnSubClass(hWnd As Long) As Boolean
  Dim lpfnOld As Long
  
  lpfnOld = GetProp(hWnd, OLDWNDPROC)
  If lpfnOld Then
    
    If SetWindowLong(hWnd, GWL_WNDPROC, lpfnOld) Then
      Call RemoveProp(hWnd, OLDWNDPROC)
      Call RemoveProp(hWnd, OBJECTPTR)

#If DEBUGWINDOWPROC Then
      ' remove the WindowProcHook reference from the collection
      m_colWPHooks.Remove CStr(hWnd)
#End If
      
      UnSubClass = True
    
    End If   ' SetWindowLong
  End If   ' lpfnOld

End Function

' Returns the specified object reference stored in the subclassed
' window's OBJECTPTR window property.
' The object reference is valid for only as long as the calling proc holds it.

Public Function GetObj(hWnd As Long) As Object
  Dim Obj As Object
  Dim pObj As Long
  pObj = GetProp(hWnd, OBJECTPTR)
  If pObj Then
    MoveMemory Obj, pObj, 4
    Set GetObj = Obj
    MoveMemory Obj, 0&, 4
  End If
End Function

Public Function WndProc(ByVal hWnd As Long, ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
  Static fOverride As Boolean
  
  Select Case uMsg
    
    ' ======================================================
    ' When the ListBox's LBS_NOINTEGRALHEIGHT isn't set, is resize, and
    ' one of its items is partially displayed, the ListBox determines it should
    ' resize itself after it receives the WM_SIZE message. When it does, it
    ' invokes another WM_WINDOWPOSCHANGING.
    
    ' So, what we're doing here is this: Catch the WM_SIZE message. If the
    ' ListBox is visible (SIZE_RESTORED, not hidden), and the lisbox's new
    ' client height is not an even multiple of it's items' height, we set a flag
    ' indicating that the next WM_WINDOWPOSCHANGING is a reslut of
    ' the ListBox setting its integral height.
    
    ' (Opinion: The cleaner method for setting integral height would obviously
    ' be before the ListBox is first sized, when WM_WINDOWPOSCHANGING
    ' is first received. But if this were the case, then we'd have a much more
    ' difficult time overriding its integral height behavior, if at all...)
    
    Case WM_SIZE
      
      If (GetWindowLong(hWnd, GWL_STYLE) And LBS_NOINTEGRALHEIGHT) = False Then
        If (wParam = SIZE_RESTORED) Then
          fOverride = (HIWORD(lParam) Mod SendMessage(hWnd, LB_GETITEMHEIGHT, 0, 0))
        End If
      End If
      
    ' ======================================================
    ' Now, if our flag is set, do the following...
    
    Case WM_WINDOWPOSCHANGING
      Dim wp As WINDOWPOS
      Dim rc As RECT
      
      If fOverride Then
        ' Our flag is set, clear it
        fOverride = False
        
        ' Get the WINDOWPOS (containing the ListBox's new integral
        ' height window rect, in ListBox parent window client coords),
        MoveMemory wp, ByVal lParam, Len(wp)
        
        ' Get the ListBox's window rect, and convert the rect to the ListBox's
        ' parent window's client coords.
        Call GetWindowRect(hWnd, rc)
        Call MapWindowPoints(HWND_DESKTOP, GetParent(hWnd), rc, 2)
        
        ' Set the WINDOWPOS coords to the ListBox's current coords
        ' (overriding the new integral height coods)
        wp.x = rc.Left
        wp.y = rc.Top
        wp.cx = rc.Right - rc.Left
        wp.cy = rc.Bottom - rc.Top
        
        ' Make sure the appropriate flags are set
        If (wp.flags And SWP_NOMOVE) Then
          wp.flags = wp.flags And Not SWP_NOMOVE
        End If
        If (wp.flags And SWP_NOSIZE) Then
          wp.flags = wp.flags And Not SWP_NOSIZE
        End If
        
        ' And finally, put the new WINDOWPOS struct data back at its
        ' memory location. The ListBox will not attempt to set the integral
        ' height in this series of resize messages again.
        MoveMemory ByVal lParam, wp, Len(wp)
      End If
      
'    ' ======================================================
'    Case WM_NOTIFY
'      Dim dwNotifyRtn As Long
'
'      dwNotifyRtn = NotificationHandler(hWnd, wParam, lParam)
'      If dwNotifyRtn Then
'        WndProc = dwNotifyRtn
'        Exit Function
'      End If
                
    ' ======================================================
    ' Unsubclass the window.
    
    Case WM_DESTROY
      ' OLDWNDPROC will be gone after UnSubClass is called!
      Call CallWindowProc(GetProp(hWnd, OLDWNDPROC), hWnd, uMsg, wParam, lParam)
      Call UnSubClass(hWnd)
      Exit Function
      
  End Select
  
  WndProc = CallWindowProc(GetProp(hWnd, OLDWNDPROC), hWnd, uMsg, wParam, lParam)
  
End Function
